
#define DMA_BUFF_SIZE 4096

/* Network statistics, with the same names as 'struct enet_statistics'. */
#define netstats enet_statistics

#define ON 1
#define OFF 0


/* 
 * Register offset info, specific to the GPCM ESCC
 * E.g., to read the data port on channel A, use
 * inportb(gpcmchan[dev].base + CHANA + DATA)
 */
#define CHANB		0	/* Base of channel B regs */
#define CHANA		2	/* Base of channel A regs */

/* 8530 ports on each channel */
#define CTL		0
#define DATA		1

#define DMAIRQEN   	0x4 	/* Offset off DMA/IRQ Enable register */
#define DMAIRQOFF	0x0	/* Disable both DMA and IRQ */
#define DMAIRQON	0x3	/* Enable both DMA and IRQ */
#define DMAON		0x2	/* Enable DMA */
#define IRQON		0x1	/* Enable IRQ */

#define DIV0		491 	/* 4915200 / 10000 (100 us units) */

/* Timer chip offsets */
#define TMR0		0x8 	/* Offset of timer 0 register */
#define TMR1		0x9 	/* Offset of timer 1 register */
#define TMR2		0xA 	/* Offset of timer 2 register */
#define TMRCMD  	0xB 	/* Offset of timer command register */

/* DMA controller registers */
#define DMA_STAT	8	/* DMA controller status register */
#define DMA_CMD	 	8	/* DMA controller command register */
#define DMA_MASK	10	/* DMA controller mask register */
#define DMA_MODE	11  	/* DMA controller mode register */
#define DMA_RESETFF 	12	/* DMA controller first/last flip flop  */

/* DMA data */
#define DMA_DISABLE 	(0x04)	/* Disable channel n */
#define DMA_ENABLE  	(0x00)	/* Enable channel n */

/* Single transfers, incr. address, auto init, writes, ch. n */
#define DMA_RX_MODE 	(0x54)

/* Single transfers, incr. address, no auto init, reads, ch. n */
#define DMA_TX_MODE 	(0x48)

#define CLOCK 4915200l

#define SIOCGGPCMPARAM	0x5000	/* get GPCM parameters */
#define SIOCSGPCMPARAM	0x5001	/* set */
#define SIOCGGPCMBAUD	0x5002	/* get only baud rate */
#define SIOCSGPCMBAUD	0x5003	
#define SIOCGGPCMDMA	0x5004	/* get only DMA */
#define SIOCSGPCMDMA	0x5005	
#define SIOCGGPCMIRQ	0x5006	/* get only IRQ */
#define SIOCSGPCMIRQ	0x5007	

struct gpcm_req  {
	int cmd;
	int speed;
	int clockmode;
#define INTERNAL 0		/* normal low speed operation */
#define EXTERNAL 1		/* external hi-speed operation */
#define NUM_CLOCK_MODES 2
	int txdelay;
	unsigned char persist;
	int slotime; 
	int squeldelay;
	int dmachan;	
	int irq;	
};

#ifdef __KERNEL__

/* Information that needs to be kept for each channel. */
struct gpcm_local {
	struct netstats stats;	/* %%%dp*/
	long open_time;		/* Useless example local info. */
	unsigned long xtal;

	struct mbuf *rcvbuf;	/* Buffer for current rx packet */
	struct mbuf *rxdmabuf1; /* DMA rx buffer */
	struct mbuf *rxdmabuf2; /* DMA rx buffer */

	int bufsiz;		/* Size of rcvbuf */
	char *rcp;		/* Pointer into rcvbuf */

	struct sk_buff_head sndq;  /* Packets awaiting transmission */
	int sndcnt;		/* Number of packets on sndq */
	struct sk_buff *sndbuf;	/* Current buffer being transmitted */
	char *txdmabuf;		/* Transmit DMA buffer */
	char *txptr;		/* Used by B port tx */
	int txcnt;			
	char tstate;		/* Transmitter state */
#define IDLE		0   	/* Transmitter off, no data pending */
#define ACTIVE  	1   	/* Transmitter on, sending data */
#define UNDERRUN	2  	/* Transmitter on, flushing CRC */
#define FLAGOUT		3   	/* CRC sent - attempt to start next frame */
#define DEFER		4 	/* Receive Active - DEFER Transmit */
#define ST_TXDELAY	5	/* Sending leading flags */
#define CRCOUT		6
	char rstate;		/* Set when !DCD goes to 0 (TRUE) */
				/* Normal state is ACTIVE if Receive enabled */
#define RXERROR 2		/* Error -- Aborting current Frame */
#define RXABORT 3		/* ABORT sequence detected */
#define TOOBIG 4		/* too large a frame to store */
	int dev;		/* Device number */
	int base;   		/* Base of I/O registers */
	int cardbase; 		/* Base address of card */
	int stata;   		/* address of Channel A status regs */
	int statb;		/* address of Channel B status regs */
	int speed;		/* Line speed, bps */
	int clockmode;		/* tapr 9600 modem clocking option */
	int txdelay;  		/* Transmit Delay 10 ms/cnt */
	unsigned char persist;	/* Persistence (0-255) as a % */
	int slotime;  		/* Delay to wait on persistence hit */
	int squeldelay;		/* Delay after XMTR OFF for squelch tail */
	struct iface *iface;	/* Associated interface */
	int dmachan;		/* DMA channel for this port */
};

#endif
